const ipc = require("node-ipc");
const {fork} = require("child_process");
const {v4: uuid4} = require("uuid");
const {rand} = require("./utils");

class Client {
	#config = {
		ipc: {  // node-ipc config
			// appspace
			// socketRoot
			id: "node-ipc-store",
			retry: 1000,
			silent: true
		},
		serverPath: "./server.js", // clients automatically start the server
		defaultRequestTtl: 5000, // requests are considered failed if the time has expired (use -1 as infinity)
		requestTtlTestInterval: 1000
	};

	#ipc;

	#requestIdPrefix = uuid4();
	#requestCount = 0;
	#requests = {};  // requestId -> eventType, details, onResult, exp

	#forkTimerId;

	get(key, onResult, ttl = this.#config.defaultRequestTtl, hash) {
		this.#send("get", {key, hash}, onResult, ttl);
	}

	set(key, value, onResult, ttl = this.#config.defaultRequestTtl) {
		this.#send("set", {key, value}, onResult, ttl);
	}

	remove(key, onResult, ttl = this.#config.defaultRequestTtl) {
		this.#send("remove", {key}, onResult, ttl);
	}

	resetCache(key, onResult, ttl = this.#config.defaultRequestTtl) {
		this.#send("resetCache", {key}, onResult, ttl);
	}

	#send(eventType, details, onResult, ttl = this.#config.defaultRequestTtl) {
		const requestId = this.#requestIdPrefix + "-" + ++this.#requestCount;
		this.#requests[requestId] = {eventType, details, onResult, exp: ttl < 0 ? -1 : new Date().getTime() + ttl};
		const message = {requestId, details};
		this.#ipc.of[this.#config.ipc.id].emit(eventType, message);
	}

	#resend() {
		for (const requestId in this.#requests) {
			const request = this.#requests[requestId];
			const message = {requestId, details: request.details};
			this.#ipc.of[this.#config.ipc.id].emit(request.eventType, message);
		}
	}

	#clear() {
		const now = new Date().getTime();
		const requestIds = [];
		for (const requestId in this.#requests) {
			if (this.#requests[requestId].exp > 0 && now > this.#requests[requestId].exp) {
				requestIds.push(requestId);
			}
		}
		for (const requestId of requestIds) {
			const request = this.#requests[requestId];
			delete this.#requests[requestId];
			request.onResult && request.onResult(false, "timeout");
		}
		setTimeout(() => this.#clear(), this.#config.requestTtlTestInterval);
	}

	constructor(config) {
		Object.assign(this.#config, config);
		Object.assign(this.#config.ipc, config && config.ipc);

		this.#ipc = new ipc.IPC;
		Object.assign(this.#ipc.config, this.#config.ipc);

		this.#ipc.connectTo(
			this.#config.ipc.id,
			() => {
				this.#ipc.of[this.#config.ipc.id].on(
					"connect",
					() => {
						console.log("Client: connected");
						this.#resend();
					}
				);
				this.#ipc.of[this.#config.ipc.id].on(
					"disconnect",
					() => {
						console.log("Client: disconnected");
						if (this.#forkTimerId === undefined) {
							this.#forkTimerId = setTimeout(() => {
								this.#forkTimerId = undefined;
								console.log("Client: fork");
								fork(this.#config.serverPath, [], {detached: true});
							}, rand(0, this.#config.retry / 2));
						}
					}
				);
				this.#ipc.of[this.#config.ipc.id].on(
					"ret",
					data => {
						const request = this.#requests[data.requestId];
						if (request === undefined) {
							return;
						}
						delete this.#requests[data.requestId];
						request.onResult && request.onResult(data.success, data.details);
					}
				);
			}
		);

		this.#clear();
	}
}

module.exports = {Client};
