const fs = require("fs");
const {Accessor} = require("./Accessor");
const {md5} = require("./utils");

const repairFileSync = path => {
	const tmp = path + ".tmp";
	if (fs.existsSync(path)) {
		if (fs.existsSync(tmp)) {
			fs.unlinkSync(tmp);
		}
	} else if (fs.existsSync(tmp)) {
		fs.renameSync(tmp, path);
	}
};

const readFileSafelySync = path => {
	repairFileSync(path);
	return fs.readFileSync(path);
};

const writeFileSafelySync = (path, content) => {
	repairFileSync(path);
	fs.writeFileSync(path, content);
};

const unlinkSafelySync = path => {
	const tmp = path + ".tmp";
	if (fs.existsSync(tmp)) {
		fs.unlinkSync(tmp);
	}
	if (fs.existsSync(path)) {
		fs.unlinkSync(path);
	}
};

class FileAccessor extends Accessor {
	#config = {
		path: "./"  // the values are stored in the path/key files
	};

	get(key, onResult) {
		let content;
		try {
			content = readFileSafelySync(this.#config.path + key);
		} catch (e) {
			onResult && onResult(false, "fs error " + e.toString());
			return;
		}
		let data;
		try {
			data = JSON.parse(content).data;
		} catch (e) {
			onResult && onResult(false, "parse error " + e.toString());
			return;
		}
		onResult && onResult(true, data);
	}

	set(key, value, onResult) {
		let content;
		try {
			content = JSON.stringify({data: value});
		} catch (e) {
			onResult && onResult(false, "stringify error " + e.toString());
			return;
		}
		try {
			writeFileSafelySync(this.#config.path + key, content);
		} catch (e) {
			onResult && onResult(false, "fs error " + e.toString());
			return;
		}
		onResult && onResult(true);
	}

	remove(key, onResult) {
		try {
			unlinkSafelySync(this.#config.path + key);
		} catch (e) {
			onResult && onResult(false, "fs error " + e.toString());
			return;
		}
		onResult && onResult(true);
	}

	hash(key, value) {
		return md5(JSON.stringify({data: value}));
	}

	constructor(config) {
		super();

		Object.assign(this.#config, config);

		if (!fs.existsSync(this.#config.path)) {
			fs.mkdirSync(this.#config.path);
		}
	}
}

module.exports = {FileAccessor, readFileSafelySync, writeFileSafelySync, unlinkSafelySync};
