const ipc = require("node-ipc");
const {Accessor} = require("./Accessor");

class Server {
	#config = {
		ipc: {  // node-ipc config
			// appspace
			// socketRoot
			id: "node-ipc-store",
			silent: true
		},
		accessor: new Accessor,  // accessor used to restore values in the cache
		cachedValueTtl: 3600000,  // values are removed from the cache if the time has expired (use -1 as infinity)
		cachedValueTtlTestInterval: 60000,
		startupTimeout: 5000  // server shuts down if the time after startup has expired and the clients have not connected (use -1 as infinity)
	};

	#ipc;

	#values = {};  // key -> value, exp
	#hashes = {};  // key -> hash

	#startupTimerId;

	#reduceValue(key, value, hash) {
		if (hash === undefined) {
			return value;
		}
		let hash_ = this.#hashes[key];
		if (hash_ === undefined) {
			hash_ = this.#config.accessor.hash(key, value);
			if (hash_ !== undefined) {
				this.#hashes[key] = hash_;
			}
		}
		return hash_ === hash ? undefined : value;
	}

	#get(key, hash, onResult) {
		const value = this.#values[key];
		if (value === undefined) {
			this.#config.accessor.get(key, (success, details) => {
				if (success) {
					this.#values[key] = {value: details, exp: this.#config.cachedValueTtl < 0 ? -1 : new Date().getTime() + this.#config.cachedValueTtl};
				}
				onResult && onResult(success, success ? this.#reduceValue(key, details, hash) : details);
			});
			return;
		}
		onResult && onResult(true, this.#reduceValue(key, value.value, hash));
	}

	#set(key, value, onResult) {
		this.#config.accessor.set(key, value, (success, details) => {
			if (!success) {
				delete this.#values[key];
				delete this.#hashes[key];
				onResult && onResult(success, details);
				return;
			}
			this.#values[key] = {value, exp: this.#config.cachedValueTtl < 0 ? -1 : new Date().getTime() + this.#config.cachedValueTtl};
			const hash = this.#config.accessor.hash(key, value);
			if (hash !== undefined) {
				this.#hashes[key] = hash;
			}
			onResult && onResult(success);
		});
	}

	#clear() {
		const now = new Date().getTime();
		const keys = [];
		for (const key in this.#values) {
			if (this.#values[key].exp > 0 && now > this.#values[key].exp) {
				keys.push(key);
			}
		}
		for (const key of keys) {
			delete this.#values[key];
			delete this.#hashes[key];
		}
		setTimeout(() => this.#clear(), this.#config.cachedValueTtlTestInterval);
	}

	constructor(config) {
		Object.assign(this.#config, config);
		Object.assign(this.#config.ipc, config && config.ipc);

		this.#ipc = new ipc.IPC;
		Object.assign(this.#ipc.config, this.#config.ipc);

		if (this.#config.startupTimeout > 0) {
			this.#startupTimerId = setTimeout(() => {
				console.log("Server: no clients - exit");
				process.exit();
			}, this.#config.startupTimeout);
		}

		this.#ipc.serve(() => {
			this.#ipc.server.on(
				"connect",
				() => {
					console.log("Server: client connected");
					if (this.#startupTimerId !== undefined) {
						clearTimeout(this.#startupTimerId);
						this.#startupTimerId = undefined;
					}
				}
			);
			this.#ipc.server.on(
				"socket.disconnected",
				() => {
					console.log("Server: client disconnected");
				}
			);
			this.#ipc.server.on(
				"get",
				(data, socket) => {
					const message = {requestId: data.requestId};
					if (data.details === undefined) {
						Object.assign(message, {success: false, details: "no details"});
						this.#ipc.server.emit(socket, "ret", message);
						return;
					}
					this.#get(data.details.key, data.details.hash, (success, details) => {
						Object.assign(message, {success, details});
						this.#ipc.server.emit(socket, "ret", message);
					});
				}
			);
			this.#ipc.server.on(
				"set",
				(data, socket) => {
					const message = {requestId: data.requestId};
					if (data.details === undefined) {
						Object.assign(message, {success: false, details: "no details"});
						this.#ipc.server.emit(socket, "ret", message);
						return;
					}
					this.#set(data.details.key, data.details.value, (success, details) => {
						Object.assign(message, {success, details});
						this.#ipc.server.emit(socket, "ret", message);
					});
				}
			);
			this.#ipc.server.on(
				"remove",
				(data, socket) => {
					const message = {requestId: data.requestId};
					if (data.details === undefined) {
						Object.assign(message, {success: false, details: "no details"});
						this.#ipc.server.emit(socket, "ret", message);
						return;
					}
					this.#config.accessor.remove(data.details.key, (success, details) => {
						delete this.#values[data.details.key];
						delete this.#hashes[data.details.key];
						Object.assign(message, {success, details});
						this.#ipc.server.emit(socket, "ret", message);
					});
				}
			);
			this.#ipc.server.on(
				"resetCache",
				(data, socket) => {
					const message = {requestId: data.requestId};
					if (data.details === undefined) {
						Object.assign(message, {success: false, details: "no details"});
						this.#ipc.server.emit(socket, "ret", message);
						return;
					}
					delete this.#values[data.details.key];
					delete this.#hashes[data.details.key];
					Object.assign(message, {success: true});
					this.#ipc.server.emit(socket, "ret", message);
				}
			);
		});

		this.#ipc.server.start();

		this.#clear();
	}
}

module.exports = {Server};
